#include <stdlib.h>
#include "treenodelib.h"
template <class T>
class BinSTree
{
   private:
      TreeNode<T> *root, *current;
      int size;
      TreeNode<T> * FindNode(const T& item,
		TreeNode<T> * &parent) const;
   public:
	  // 
      BinSTree(void): root(NULL), current(NULL), size(0){}
	  //  
      BinSTree(const BinSTree<T> & tree);
	  // 
      ~BinSTree(void) {DeleteTree(root);}
	  //   
      BinSTree<T>& operator=(const BinSTree<T> & tree);
      //  
      int Find(T& item);
	  //    
      void Insert(const T & item);
      //    
      void Delete(const T & item);
	  //  
      void ClearTree(void){DeleteTree(root);
									root=current = NULL;size=0;}
      //  ?
      int TreeEmpty(void) const {return !size;}
	  //  
      int TreeSize(void) const {return size;}
      // 
      void Update(const T & item);
	  //    :
	  TreeNode<T> * GetRoot(void) {return root;}
};

template <class T>
TreeNode<T> * BinSTree<T>::FindNode(const T& item,
  		                    TreeNode<T> * &parent) const
{
   TreeNode<T> *t = root;
   parent = NULL;
   while(t)
   {
      if (item==t->data) break;
      else
      {
      	parent = t;
         if (item < t->data) t = t->left;
         else t = t->right;

      }
   }
   return t;
}

template <class T>
BinSTree<T>::BinSTree(const BinSTree<T> & tree)
{
   root = CopyTree(tree.root);
   current = root;
   size = tree.size;
}

template <class T>
BinSTree<T>& BinSTree<T>::
operator=(const BinSTree<T> & tree)
{
   DeleteTree(root);
   root = CopyTree(tree.root);
   current = root;
   size = tree.size;
   return *this;
}

template <class T>
int BinSTree<T>::Find(T& item)
{
   TreeNode<T> *parent;
   current = FindNode(item,parent);
   if (current)
   {
      item = current->data;
      return 1;
   }
   else return 0;
}

template <class T>
void BinSTree<T>::Insert(const T& item)
{
   // t -  , parent - ,
   //newNode - 
   TreeNode<T> *t = root, *parent=NULL,
   	       *newNode = new TreeNode<T>(item);
   while(t) //    
   {
      //   parent     
	  //     
		parent = t;
      if (item < t->data)
        	t = t->left;
      else
        	t = t->right;
   }
   //   ,    
   if (!parent)
     	root = newNode;
   //  item   ,   
   else if (item < parent->data)
    		parent ->left = newNode;
    	else
			//  
           	parent->right = newNode;
   //       size
   current = newNode;
   size++;
}

template <class T>
void BinSTree<T>::Delete(const T& item)
{
   // DNodePtr -     D
   // PNodePtr -    P   D
   // RNodePtr -     R
   TreeNode<T> * DNodePtr, *PNodePtr, *RNodePtr;
   //  ,     item.
   //       
   DNodePtr = FindNode(item,PNodePtr);
   //     ,   
   if (!DNodePtr) return;
   //  D  NULL-,  
   //  ,     
   if (!DNodePtr->right)
   	  RNodePtr = DNodePtr->left;
   else  if (!DNodePtr->left)
        RNodePtr = DNodePtr->right;
         else
			//  D   
         {
				//PofRNodePtr -    PR  R
				TreeNode<T> * PofRNodePtr = DNodePtr;
				//      D
            RNodePtr = DNodePtr->left;
		   	//  "",    
            while (RNodePtr->right)
            {
               PofRNodePtr = RNodePtr;
               RNodePtr = RNodePtr->right;
            }
            if (PofRNodePtr == DNodePtr)
            //    D  
            //     D  R
            	RNodePtr->right = DNodePtr->right;
            else
            {
            	//     R
					//   
					PofRNodePtr->right = RNodePtr->left;
					//   R   D
         		RNodePtr->left =  DNodePtr->left;
           		RNodePtr->right =  DNodePtr->right;
            }
         }
   //    R   P
   if (!PNodePtr)
  		//   D   -   
		//    -  R
      	root = RNodePtr;
   //      R   P?
   else  if (DNodePtr->data < PNodePtr->data)
		// 
     		PNodePtr->left = RNodePtr;
     	 else
			// 
			PNodePtr->right = RNodePtr;
   //   D      
   delete DNodePtr;
   size--;
}

template <class T>
void BinSTree<T>::Update(const T& item)
{
   if (current && current->data==item)
   	current->data = item;
   else
   	Insert(item);
}

